#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Unit tests for pyes.  These require an es server with thrift plugin running on the default port (localhost:9500).
"""
from pyestest import ESTestCase

#if __name__ == "__main__":
#    unittest.main()
#    suite = unittest.TestLoader().loadTestsFromTestCase(GeoQuerySearchTestCase)
#    suite = unittest.TestLoader().loadTestsFromTestCase(IndexingTestCase)
#    suite = unittest.TestLoader().loadTestsFromTestCase(BulkTestCase)
#    suite = unittest.TestLoader().loadTestsFromTestCase(QueryAttachmentTestCase)
#    suite = unittest.TestLoader().loadTestsFromTestCase(QuerySearchTestCase)
#    suite = unittest.TestLoader().loadTestsFromTestCase(SearchTestCase)
#    unittest.TextTestRunner(verbosity=2).run(suite)